@@ -7,7 +7,7 @@ from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
 from electrum.logging import get_logger
 from electrum.i18n import _
 from electrum.transaction import PartialTxOutput, PartialTransaction, Transaction, TxOutpoint
-from electrum.util import NotEnoughFunds, profiler
+from electrum.util import NotEnoughFunds, profiler, quantize_feerate
 from electrum.wallet import CannotBumpFee, CannotDoubleSpendTx, CannotCPFP, BumpFeeStrategy
 from electrum.plugin import run_hook
 
@@ -719,7 +719,6 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
 
         self._input_amount = QEAmount()
         self._output_amount = QEAmount()
-        self._fee_for_child = QEAmount()
         self._total_fee = QEAmount()
         self._total_fee_rate = 0
         self._total_size = 0
@@ -754,17 +753,6 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
             self._total_fee_rate = totalfeerate
             self.totalFeeRateChanged.emit()
 
-    feeForChildChanged = pyqtSignal()
-    @pyqtProperty(QEAmount, notify=feeForChildChanged)
-    def feeForChild(self):
-        return self._fee_for_child
-
-    @feeForChild.setter
-    def feeForChild(self, feeforchild):
-        if self._fee_for_child != feeforchild:
-            self._fee_for_child.copyFrom(feeforchild)
-            self.feeForChildChanged.emit()
-
     inputAmountChanged = pyqtSignal()
     @pyqtProperty(QEAmount, notify=inputAmountChanged)
     def inputAmount(self):
@@ -850,12 +838,12 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
         comb_fee = fee + self._parent_fee
         comb_feerate = comb_fee / self._total_size
 
-        self._fee_for_child.satsInt = fee
+        self._fee.satsInt = fee
         self._output_amount.satsInt = self._max_fee - fee
         self.outputAmountChanged.emit()
 
         self._total_fee.satsInt = fee + self._parent_fee
-        self._total_fee_rate = f'{comb_feerate:.1f}'
+        self._total_fee_rate = str(quantize_feerate(comb_feerate))
 
         try:
             self._new_tx = self._wallet.wallet.cpfp(self._parent_tx, fee)
@@ -864,6 +852,9 @@ class QETxCpfpFeeBumper(TxFeeSlider, TxMonMixin):
             self.warning = str(e)
             return
 
+        child_feerate = fee / self._new_tx.estimated_size()
+        self.feeRate = str(quantize_feerate(child_feerate))
+
         self.update_inputs_from_tx(self._new_tx)
         self.update_outputs_from_tx(self._new_tx)
 
